@@ -11,12 +11,14 @@ use thiserror::Error;
 
 #[derive(Debug, Error)]
 pub enum Error {
-    #[error("Failed to write multipart header: {0:?}")]
-    HeaderWrite(IoError),
-
-    #[error("Failed to write multipart boundary: {0:?}")]
-    BoundaryWrite(IoError),
-
     #[error("Failed to write multipart content: {0:?}")]
     ContentRead(IoError),
 }
+
+impl Into<IoError> for Error {
+    fn into(self) -> IoError {
+        match self {
+            Error::ContentRead(io) => io,
+        }
+    }
+}
